@@ -2,7 +2,6 @@ import shopify
 from singer.utils import strftime, strptime_to_utc
 from tap_shopify.context import Context
 from tap_shopify.streams.base import (Stream,
-                                      RESULTS_PER_PAGE,
                                       shopify_error_handling,
                                       OutOfOrderIdsError)
 
@@ -13,9 +12,11 @@ class OrderRefunds(Stream):
 
     @shopify_error_handling
     def get_refunds(self, parent_object, since_id):
+        # set timeout
+        self.replication_object.set_timeout(self.request_timeout)
         return self.replication_object.find(
             order_id=parent_object.id,
-            limit=RESULTS_PER_PAGE,
+            limit=self.results_per_page,
             since_id=since_id,
             order='id asc')
 
@@ -33,7 +34,7 @@ class OrderRefunds(Stream):
                         raise OutOfOrderIdsError("refund.id < since_id: {} < {}".format(
                             refund.id, since_id))
                     yield refund
-                if len(refunds) < RESULTS_PER_PAGE:
+                if len(refunds) < self.results_per_page:
                     break
                 if refunds[-1].id != max([o.id for o in refunds]):
                     raise OutOfOrderIdsError("{} is not the max id in refunds ({})".format(
